/**********************************************************************************/
/* INFOMAPS procedure and LIBNAME engine demonstration code for SAS 9.2.          */
/* Author: Bill McNeill                                                           */
/* Date:   February 22, 2009                                                      */
/*                                                                                */
/* Demonstration of code detailed in SAS Global Forum paper,                      */
/* "Test-Driving the Improvements to the INFOMAPS Procedure and LIBNAME Engine",  */
/* Paper #301-2009. Available at:                                                 */
/* http://support.sas.com/saspresents                                             */
/**********************************************************************************/

/*************************************************************************/
/*                                                                       */
/*   Setting up a SAS libref to the physical data                        */
/*                                                                       */
/*************************************************************************/

/* The SAS Management Console is needed to define a SAS library. This    */
/* SAS library is where the physical data resides that will be used to   */
/* create the information map. The SAS Management Console will store     */
/* this library definition on a metadata server. This demo assumes that  */
/* an installed version of SAS V9.2 for Windows is being used.           */
/* Therefore, the location of the physical data (in this case, SAS data  */
/* sets) used to define the information map are located on your local    */
/* machine at the path:                                                  */
/*                                                                       */
/*     C:\Program Files\SAS\SASFoundation\9.2\core\sample                */
/*                                                                       */
/* The SAS data set names are:                                           */
/*                                                                       */
/*    empinfo.sas7bcat                                                   */
/*    jobcodes.sas7bcat                                                  */
/*    salary.sas7bcat                                                    */
/*                                                                       */
/* Once SAS Management Console 9.2 is started and you are connected to   */
/* your metadata server, expand the following in the left hand side      */
/* tree view window:                                                     */
/*                                                                       */
/*    Environment Management -> Data Library Manager -> Libraries        */
/*                                                                       */
/* Activate the pop-up menu using the right mouse button while hovering  */
/* over "Libraries" in the tree view and click on the "New Library..."   */
/* selection. This will bring up the New Library Wizard. Select          */
/* "SAS BASE Library" from the tree view. If the tree view is not        */
/* already expanded, the path to the selection is:                       */
/*                                                                       */
/*     Resource Template -> Libraries -> SAS Data -> SAS BASE Library    */
/*                                                                       */
/* In the next window, you will be asked to supply the Name,             */
/* Description, and Location of the library being created. The Library   */
/* name entered is what will be used in the INFOMAPS procedure. The      */
/* description can be anything you feel appropriate. The location is the */
/* folder where the library will live in the metadata server. For the    */
/* demo, the following is used:                                          */
/*                                                                       */
/*     Name: HR                                                          */
/*     Description: Library for SAS 9.2 INFOMAPS demo.                   */
/*     Location: /Shared Data/Demo_V92_Doc                               */
/*                                                                       */
/* Advance to the next wizard window by clicking the "Next >" button.    */
/* This will put you on the SAS Servers window. You should be able to    */
/* select the SASMain server and use the right arrow icon to move that   */
/* value to the "Selected servers:" frame.                               */
/*                                                                       */
/* The "Next >" button will take you to the Library Properties window.   */
/* In the Libref field, enter a valid SAS libname (limited to 8          */
/* alphanumerics). The Engine field should already contain the desired   */
/* engine, BASE. The "Enable WebDAV Support" check box needs to remain   */
/* unchecked. Click the "New..." button to add the path specification to */
/* the physical data. Since the data is local to my machine, I just      */
/* entered the path into the Paths field. For the demo, the following is */
/* used:                                                                 */
/*                                                                       */
/*  Libref: V92_Demo                                                     */
/*  Engine: BASE                                                         */
/*  Path: C:\Program Files\SAS\SASFoundation\9.2\core\sample             */
/*                                                                       */
/* The final window in the wizard is the confirmation screen. If         */
/* everything looks correct, click the Finish button. The library has    */
/* been added to the metadata.                                           */
/*                                                                       */
/* You can now submit the demonstration code in SAS.                     */ 


/*************************************************************************/
/*                                                                       */
/*  Assigning the Common Connection Parameters                           */
/*                                                                       */
/*************************************************************************/

/* Set common settings for running these programs. */
/* Set system options for connecting to the metadata server. */

/********************************** NOTE *********************************/
/*                                                                       */
/*                                                                       */
/* You need to supply values for the 4 options and a macro variable:     */
/* METAUSER, METAPASS, METASERVER, METAPORT, and INFOMAP_PATH. Each has  */
/* a description of the needed value contained in angle brackets, <>.    */
/*                                                                       */
/*                                                                       */
options metauser="<metadata username>"
        metapass="<metadata password>"
        metaserver="<metadata server name>"
        metaport=<metadata port number>;
%LET infomap_path=<path to infomation map within metadata repository>;
/*************************************************************************/

%LET infomap_name=Employee Info;

/*************************************************************************/
/*                                                                       */
/*  Importing Metadata from the Data Sources                             */
/*                                                                       */
/*************************************************************************/

/* SAS Code to populate metadata about data sources. */
proc metalib;
     omr (library="HR");                     /* library name             */
     select("empinfo" "jobcodes" "salary");  /* data source table names  */
     /* Create a summary report of the metadata changes. */
     report;
     run;

/*************************************************************************/
/*                                                                       */
/*  Initial Creation of the Information Map                              */
/*                                                                       */
/*************************************************************************/
proc infomaps mappath="&infomap_path";
/* Open a new information map or replace it if it exists. */
new infomap "&infomap_name"
    auto_replace=yes;

insert datasource 
       sasserver="SASMain"
       table="HR".empinfo
       _all_
       id="Empinfo";

list;  /* List the current contents of the information map */
save;  /* Save the information map.                        */
run;   /* End the INFOMAPS procedure.                      */

/* Compare the information map to the original data source */

/* Assign libref to original data sources. */
libname HR base "C:\Program Files\SAS\SASFoundation\9.2\core\sample";
/* Show the original data source. */
proc contents data=hr.empinfo; 
run;

/* Use the INFOMAPS LIBNAME engine */
libname emp infomaps mappath="&infomap_path";
/* list the available maps */
proc datasets lib=emp;
run;
/* display the contents of the information map */
proc contents data=emp."&infomap_name"n; 
run;

/* Use the ValidVarName option to display non-SAS normal names. */
option validvarname=any;
proc contents data=emp."&infomap_name"n; 
run;

/*************************************************************************/
/*                                                                       */
/*   Update Existing Data Item                                           */
/*                                                                       */
/*************************************************************************/

/* 
   Update the second "Address" data item to have a unique name within
   the information map. Also update the description to be more 
   descriptive. 
*/
proc infomaps mappath="&infomap_path";
update infomap "&infomap_name";
update dataitem "Address_2"
		name="Address 2"
		description="Address line #2";
save;  /* Save the information map.                        */
run;   /* End the INFOMAPS procedure.                      */

/* View the updates to the information map. */
proc contents data=emp."&infomap_name"n; 
run;

/*************************************************************************/
/*                                                                       */
/*   Adding Data Sources                                                 */
/*                                                                       */
/*************************************************************************/

/* Add more data sources to the information map. */
proc infomaps mappath="&infomap_path";
update infomap "&infomap_name";

/* 
    Insert a second data source, but put only 1 data item into the 
    current information map. Note that this data item is created at 
    the root level of the information map. 
*/
insert datasource sasserver="SASMain"
       table="HR".jobcodes
       columns=( "TITLE" )
       id="Jobcodes" /* force case of ID name; default is JOBCODES */
       desc="Job Code Information";

/* Define how the tables are joined. */
insert relationship
       left_table="Empinfo"
       right_table="Jobcodes"
       join=inner
       condition="(<<Empinfo.JOBCODE>>=<<Jobcodes.JOBCODE>>)";

/* 
	Insert a third data source into the current information map. 
	Since neither the _ALL_ nor COLUMNS option was specified,  
	no data items are inserted into the information map. 
*/
insert datasource sasserver="SASMain"
       table="HR".salary
       id="Salary";

/* Define how the tables are joined. */
insert relationship
       left_table="Empinfo"
       right_table="Salary"
       join=inner
       condition="(<<Empinfo.IDNUM>>=<<Salary.IDNUM>>)"
       desc="Employee Salary Information";

list datasources;   /* List the data sources in the information map. */
list relationships; /* List the ways the data sources are joined.    */
save;               /* Save the information map.                     */
run;                /* End the INFOMAPS procedure.                   */


/*************************************************************************/
/*                                                                       */
/*  Folders in the Metadata                                              */
/*                                                                       */
/*************************************************************************/

/* Insert a folder and additional data items into the folder. */
proc infomaps mappath="&infomap_path";
update infomap "&infomap_name";
insert folder "Salary Info";

insert dataitem column="Salary".salary
       name="Annual Salary" folder="Salary Info";

insert dataitem column="Salary".enddate folder="Salary Info";

save;  /* Save the information map.                        */
run;   /* End the INFOMAPS procedure.                      */


/* View the current information map. */
proc contents data=emp."&infomap_name"n; 
run;

/*************************************************************************/
/*                                                                       */
/*  Adding Filters                                                       */
/*                                                                       */
/*************************************************************************/

/* Define and insert filters into the information map. */
proc infomaps mappath="&infomap_path";
update infomap "&infomap_name";

/* Insert 4 filters */
insert filter "Status is Current"
       condition="<<root.Enddate>> IS NULL" folder="Salary Info"
       desc="Currently employed";

insert filter "Education and Publications Departments"
       condition='SUBSTRN(<<root.Jobcode>>, 1, 3) IN ("EDU","PUB")'
       desc="Employees in Education and Publications";

insert filter "Host Systems Development"
       condition='<<root.Division>>="HOST SYSTEMS DEVELOPMENT" '
       desc="Employees in Host Systems Development";

insert filter "Cary HQ"
       condition='<<root.Location>>="Cary" '
       desc="Located in Cary, North Carolina HQ";

list filters; /* Lists the filters in the information map. */
save;         /* Save the information map.                 */
run;          /* End the INFOMAPS procedure.               */


/* View the current information map. */
proc contents data=emp."&infomap_name"n; 
run;

/*************************************************************************/
/*                                                                       */
/*  Moving objects within the information map                            */
/*                                                                       */
/*************************************************************************/

/* Move items within the information map. */
proc infomaps mappath="&infomap_path";
update infomap "&infomap_name";

/* Move 3 filters into a newly created folder. */
move filter id_list=("Education and Publications Departments" 
                     "Host Systems Development"
                     "Cary HQ") 
            new_location="Filters"/create; 

/* Move a data item to an existing folder. */
move dataitem "Title" new_location="Empinfo";
 
save;  /* Save the information map.                        */
run;   /* End the INFOMAPS procedure.                      */


/*************************************************************************/
/*                                                                       */
/*   Example of WHERE statement and filter option                        */
/*                                                                       */
/*************************************************************************/

/* Restrict the data returned by the information map using a filter and  */
/* a SAS Where statement. */
title "Job Code HSD007 in the Host Systems Development Division";
proc print data=emp."&infomap_name"n(filter='Host Systems Development'n); 
     where jobcode="HSD007";
run;

/*************************************************************************/
/*                                                                       */
/*  Data Items with Aggregate Function or Expressions                    */
/*                                                                       */
/*************************************************************************/

/* Insert a data item that contains an expression */
proc infomaps mappath="&infomap_path";
update infomap "&infomap_name";

/* Insert a data item that computes the monthly salary. */
insert dataitem expression="<<Salary.SALARY>>/12" 
       type=numeric
       name="Monthly Salary" folder="Salary Info"
       desc="Monthly salary computed from the yearly salary"
       classification=measure
       format="dollar12.2";

/* Update a data item to use a different default aggregation. */
update dataitem "Jobcode"
		name="Jobcode"
		aggregation=countdistinct
                classification=measure;

save;  /* Save the information map.                        */
run;   /* End the INFOMAPS procedure.                      */


/* View the aggregate information within the information map. */
title;
proc contents data=emp."&infomap_name"n(aggregate=yes); 
run;

/* Use the default aggregate funtions. */
title 'Number of unique job codes assigned to employees';
proc print data=emp."&infomap_name"n(aggregate=yes keep=jobcode); 
run;

title "Show the Salary and Monthly Salary values";
proc print data=emp."&infomap_name"n(obs=5 keep='Annual Salary'n 
                                                'Monthly Salary'n); 
run;

/*************************************************************************/
/*                                                                       */
/*  Complex Filter Usage                                                 */
/*                                                                       */
/*************************************************************************/

/* Show how filters can be used in a complex FILTER= option. */
title "Total Annual Salary for Divisions outside Host Systems and HQ";
proc print data=emp.'Employee Info'n 
               (keep='Annual Salary'n Division 
                aggregate=yes 
                filter=(    'Status is Current'n  
                         and ( not(    'Host Systems Development'n 
                                    or 'Cary HQ'n))) );
run;

/*************************************************************************/
/*                                                                       */
/*  Surfacing Information Map Details via SQL Dictionary Tables          */
/*                                                                       */
/*************************************************************************/

/* Need a new libref with the aggregate libname option turned on.        */
libname sqldict infomaps mappath="&infomap_path" 
                         aggregate=yes;

/* Demonstrate the new SQL dictionary tables based on information maps. */
/* INFOMAPS dictionary table */
proc sql;
title "SQL Dictionary Table INFOMAPS";
select * from dictionary.infomaps where libname="SQLDICT" 
                                    and memname="Employee Info";

/* DATAITEMS dictionary table */
title "SQL Dictionary Table DATAITEMS";
select * from dictionary.dataitems where libname="SQLDICT" 
                                     and memname="Employee Info"
                                     and name="Jobcode";

/* FILTERS dictionary table */
title "SQL Dictionary Table FILTERS";
select * from dictionary.filters where libname="SQLDICT" 
                                   and memname="Employee Info"
                                   and name="Cary HQ";
quit;

/*************************************************************************/
/*                                                                       */
/*  Deleting Information Map                                             */
/*                                                                       */
/*************************************************************************/
/* optionally delete the information map */
/*
proc infomaps mappath="&infomap_path";
     delete infomap "&infomap_name";
run;
*/
